#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import sys
import tests.config as cfg
import pytest

from taz.table import Table, TableBatch
from taz.storage_account import StorageAccount
from taz.exception import TableBatchExceedCapacityException


class TableTests(unittest.TestCase):
    def setUp(self):
        self.storage_account = self.storage_account = StorageAccount(
            connection_string=cfg.tables["storage_connection_string"]
        )
        self.table_name = cfg.tables["table_name"]
        self.table = Table(self.table_name, storage_account=self.storage_account)
        self.entity1 = {
            "PartitionKey": "part1",
            "RowKey": "row1",
            "field1": "val1",
        }
        self.entity2 = {
            "PartitionKey": "part1",
            "RowKey": "row2",
            "field1": "val2",
        }
        self.entity3 = {
            "PartitionKey": "part2",
            "RowKey": "row1",
            "field1": "val3",
        }
        self.entity4 = {
            "PartitionKey": "part1",
            "RowKey": "row2",
            "field2": "val4",
        }
        self.entity5 = {
            "PartitionKey": "part1",
            "RowKey": "row3",
            "field2": "val4",
        }

    def test_010_list(self):
        for table in self.storage_account.list_tables():
            print(table.name)

    def test_030_create(self):
        self.assertTrue(self.table.create().exists())

    def test_040_exists(self):
        self.assertTrue(self.table.exists())

    def test_050_insert(self):
        self.table.insert_entity(self.entity1)
        self.table.insert_entity(self.entity2)

    def test_060_upsert(self):
        self.entity1["field1"] = "val1_mod"
        self.table.upsert_entity(self.entity1)
        self.table.upsert_entity(self.entity3)

    def test_070_merge(self):
        self.table.merge_entity(self.entity4)

    def test_080_get_entity(self):
        self.assertEqual(self.table.get_entity("part1", "row2")["field1"], "val2")
        self.assertEqual(self.table.get_entity("part1", "row2")["field2"], "val4")
        self.assertEqual(self.table.get_entity("part1", "row1")["field1"], "val1_mod")
        self.assertEqual(self.table.get_entity("part2", "row1")["field1"], "val3")

    def test_090_delete_entity(self):
        (
            self.table.delete_entity("part1", "row1")
            .delete_entity("part1", "row2")
            .delete_entity("part2", "row1")
        )

    def test_100_batch_insert_exceed_capacity(self):
        batch = TableBatch(
            self.table_name,
            storage_account=self.storage_account,
            max_tasks=2,
            auto_commit=False,
        )
        with pytest.raises(TableBatchExceedCapacityException):
            (
                batch.insert_entity(self.entity1)
                .insert_entity(self.entity2)
                .insert_entity(self.entity3)
                .commit_batch()
            )

    def test_110_batch_insert(self):
        batch = TableBatch(self.table_name, storage_account=self.storage_account)
        (
            batch.insert_or_replace_entity(self.entity1)
            .insert_or_replace_entity(self.entity2)
            .insert_or_replace_entity(self.entity5)
            .commit_batch()
        )
        (
            batch.delete_entity(partition_key="part1", row_key="row1")
            .delete_entity(partition_key="part1", row_key="row2")
            .delete_entity(partition_key="part1", row_key="row3")
            .commit_batch()
        )
        batch.save_stats()

    def test_120_delete(self):
        self.assertFalse(self.table.delete().exists())


if __name__ == "__main__":
    sys.argv.append("-v")
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Taz library: containers and blobs
"""
import urllib
from azure.core.exceptions import ResourceExistsError

from azure.storage.blob import (
    BlobServiceClient,
    BlobClient,
    ContainerClient,
    BlobType,
    BlobSasPermissions,
    generate_blob_sas,
)
from datetime import datetime, timedelta
import pandas as pd
import gzip

from taz.storage_account import StorageAccount
from taz.exception import BlobAthenticationFailedException


class Container:
    def __init__(self, storage_account, name):
        """Blob Container

        :param storage_account: storage account
        :type storage_account: taz.storage_account.StorageAccount
        :param name: container name
        :type name: str
        """
        self.storage_account = storage_account
        self.name = name

        try:
            self.client = self.storage_account.blob_service_client.create_container(
                self.name
            )
        except ResourceExistsError:
            self.client = self.storage_account.blob_service_client.get_container_client(
                self.name
            )

    def get_properties(self):
        """get container properties

        :return: container properties
        :rtype: azure.storage.blob.ContainerProperties
        """
        return self.client.get_container_properties()

    def delete(self):
        """delete Container

        :return: self obj
        :rtype: taz.blob.Container
        """
        self.storage_account.blob_service_client.delete_container(self.name)
        return self

    def list_blobs(self, prefix=None):
        """list blobs in container

        :param prefix: prefix of blobs to list, defaults to None
        :type prefix: str, optional
        :return: list of blobs properties
        :rtype: Iterable of azure.storage.blob.BlobProperties
            BloProperties fields:
            - name
            - container
            - metadata
            - size
            - last_modified
            (see Azure documentation for further details) 
        """
        return self.client.list_blobs(name_starts_with=prefix)


class Blob:
    def __init__(
        self,
        container,
        name,
        sas_url=None,
        sas_key=None,
        storage_account_name=None,
        storage_account=None,
    ):
        """define blob object
        
        :param container: container name
        :type container: str
        :param name: blob name
        :type name: str
        :param sas_url: sas url, defaults to None
        :type sas_url: str, optional
        :param sas_key: sas key (needs storage_account_name), defaults to None
        :type sas_key: str, optional
        :param storage_account_name: storage account name, defaults to None
        :type storage_account_name: str, optional
        :param storage_account: storage account object, defaults to None
        :type storage_account: taz.blob.StorageAccount, optional
        """
        self.storage_account = storage_account
        self.container = container
        self.name = name
        self.sas_url = sas_url
        self.storage_account = storage_account
        self.sas_key = sas_key
        self.storage_account_name = storage_account_name

        if self.storage_account:
            self.auth_mode = "storage_account_connection_string"
            self.client = self.storage_account.blob_service_client.get_blob_client(
                self.container, self.name
            )
        elif self.sas_url:
            self.auth_mode = "sas_url"
            self.client = BlobServiceClient(sas_url).get_blob_client(
                self.container, self.name
            )
        elif self.sas_key and self.storage_account_name:
            self.auth_mode = "sas_key"
            self.client = BlobServiceClient(
                "https://{}.blob.core.windows.net/{}".format(
                    self.storage_account_name, self.sas_key
                )
            ).get_blob_client(self.container, self.name)
        else:
            raise (BlobAthenticationFailedException(self.name))

    def get_sas_token(
        self, expiry=datetime.utcnow() + timedelta(hours=1), permission_string="racwd",
    ):
        """generate sas token for blob

        :param expiry: token expiration date, defaults to datetime.utcnow()+timedelta(hours=1)
        :type expiry: datetime.datetime, optional
        :param permission_string: blob access permissions, defaults "racwd" (read, add, create, write, delete)
        :type permission_string: str, optional
        :return: sas token
        :rtype: str
        """

        permission = BlobSasPermissions.from_string(permission_string)

        self.sas_token = generate_blob_sas(
            self.storage_account.name,
            self.container,
            self.name,
            account_key=self.storage_account.key,
            permission=permission,
            expiry=expiry,
        )
        return self.sas_token

    def get_url(
        self, expiry=datetime.utcnow() + timedelta(hours=1), permission_string="racwd",
    ):
        """get url with sas token to download blob

        :param expiry: token expiration date, defaults to datetime.utcnow()+timedelta(hours=1)
        :type expiry: datetime.datetime, optional
        :param permission: blob access permissions, defaults "racwd" (read, add, create, write, delete)
        :type permission: BlobSasPermissions, optional
        :return: blob url
        :rtype: str
        """
        return "{}?{}".format(
            self.client.url,
            self.get_sas_token(expiry=expiry, permission_string=permission_string),
        )

    def read(self):
        """read datas from blob
        
        datas neads to be decoded, example for utf8 text:
        myblob.read().decode("utf8")

        :return: binary data read
        :rtype: bytes
        """
        return self.client.download_blob().readall()

    def read_csv(self, **kargs):
        """read CSV file from Blob

        :param **kargs: all pandas.Dataframe.read_csv() params
        :type kargs: *
        :return: DataFrame filled with read datas
        :rtype:  pandas.DataFrame
        """
        return pd.read_csv(self.get_url(), **kargs)

    def gzip_write(self, data, overwrite=True):
        """write gzipped data to blob
        
        :param data: datas to write
        :type data: binary string
        :param overwrite: overwrite or not, defaults to True
        :type overwrite: bool, optional
        """
        return self.write(gzip.compress(data, compresslevel=9), overwrite=overwrite)

    def write(self, data, overwrite=True):
        """write data to blob

        :param data: data
        :type data: bytes
        :param overwrite: overwrite blob if existing, defaults to True
        :type overwrite: bool, optional
        :return: self obj
        :rtype: taz.blob.Blob
        """
        self.client.upload_blob(data, overwrite=overwrite)
        return self

    def upload(self, path, overwrite=True):
        """upload file to blob
        
        :param path: local path
        :type path: str
        :param overwrite: overwrite blob or not, defaults to True
        :type overwrite: bool, optional
        """
        with open(path, "rb") as data:
            self.write(data, overwrite=True)
        return self

    def download(self, path):
        """download blob to local file

        :param path: local path to write blob
        :type path: str
        :return: self obj
        :rtype: taz.blob.Blob
        """
        with open(path, "wb") as data:
            data.write(self.read())
        return self

    def delete(self):
        """delete blob

        :return: self obj
        :rtype: taz.blob.Blob
        """
        self.client.delete_blob()
        return self

    def add_metadata(self, metadata):
        """merge metadata to existing

        :param metadata: metadata to merge
        :type metadata: dict
        :return: self obj
        :rtype: taz.blob.Blob
        """
        new_metadata = self.get_metadata()
        new_metadata.update(metadata)
        self.set_metadata(new_metadata)
        return self

    def set_metadata(self, metadata: dict):
        """replace blob metadata

        :param metadata: metadata (if None, reset)
        :type metadata: dict
        """
        self.client.set_blob_metadata(metadata)
        return self

    def get_metadata(self):
        """get blob metadata

        :return: metadatas
        :rtype: dict
        """
        return self.get_properties().metadata

    def get_properties(self):
        """get all blob properties

        :return: blob proberties object
        :rtype: azure.storage.blob.BlobProperties
            BloProperties fields:
            - name
            - container
            - metadata
            - size
            - last_modified
            (see Azure documentation for further details) 
        """
        return self.client.get_blob_properties()

    def get_size(self):
        """get blob size

        :return: blob size (bytes)
        :rtype: int
        """
        return self.client.get_blob_properties().size



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from azure.common.client_factory import get_client_from_cli_profile
from azure.mgmt.containerregistry import ContainerRegistryManagementClient
from taz.acr import ContainerRegistry
from msrestazure.azure_active_directory import MSIAuthentication
from azure.mgmt.containerinstance.container_instance_management_client import (
    ContainerInstanceManagementClient,
)

from azure.mgmt.containerinstance.models import (
    ContainerGroup,
    Container,
    ContainerGroupNetworkProtocol,
    ContainerPort,
    EnvironmentVariable,
    IpAddress,
    Port,
    ResourceRequests,
    ResourceRequirements,
    OperatingSystemTypes,
)

import json
import os
from azure.common.credentials import ServicePrincipalCredentials
from datetime import datetime

from taz.auth import UserAssignedIdentity, ClientSecretAuthentication, GenericClient
from taz.exception import ContainerAlreadyExistsException
from taz.blob import Blob


class SimpleContainerGroup:

    """Simplified Container Group with a Single container
    
    Attributes:
        - name (str): Description
        - generic_client (taz.auth.GenericClient): generic client to abstract authentication method
        - client (azure.mgmt.containerinstance.container_instance_management_client.ContainerInstanceManagementClient): 
          ACI management client provided by generic_client
        - command (list(str)): container entry point
        - cpus (int): vcpus for all container group (1-4 for northeurope 2020/03)
        - env_vars (list[azure.mgmt.containerinstance.models.EnvironmentVariable]): container environment variables
        - group (ContainerGroup): container group
        - identity (taz.auth.UserAssignedIdentity): user managed identity
        - image (taz.acr.ContainerImage): docker image to use
        - instances (list[azure.mgmt.containeris-nstance.Container]): Container instances in container group
        - location (str): Azure location
        - mem (int): memory for all container group (1-14GB for northeurope 2020/03)
        - os_type (constant, str): set to OperatingSystemTypes.linux
        - resource_group_name (str): resource group
        - restart_policy (str): Always/OnFailure/Never
        - tags (dict[str, str]): Azure tags
        - state (string): state of container group 
        (Accessed if already created, Provisionned if it is prepared, and Created if it is created)
        - protected (bool): display env vars values or not
        - identity_name (str): name of user managed identity
    """

    def __init__(
        self,
        name,
        resource_group_name,
        location="northeurope",
        image=None,
        subscription_id=None,
        tags=None,
        cpus=1,
        mem=1,
        command=None,
        identity_name=None,
        identity=None,
        restart_policy="Never",
        env_vars=[],
    ):
        """
        Simple container group object with one instance
        
        Args:
            - name (str): container group name
            - resource_group_name (str): resource group
            - location ("northeurope", str, optionnal): location
            - image (None, taz.acr.ContainerImage, optionnal): ContainerImage object (if no image, get existing container group)
            - subscription_id (None, optional, str): subscription id
            - tags (None, optionnal, dict[str, str]): tags 
            - cpus (1, int, optionnal): vcpus for all container group (1-4 for northeurope 2020/03)
            - mem (1, int, optionnal): memory for all container group (1-14GB for northeurope 2020/03)
            - command (None, list(str), optionnal): container entry point
            - identity (None, taz.auth.UserAssignedIdentity, optionnal): User managed identity to affect
            - restart_policy ("Never", str, optionnal): set restart policy of container
            - env_vars (None, dict, optionnal):  environment variables dictionnary to set
            - identity_name (None, str, optionnal): user managed identity by name (identity parameter ignored if set)
        """

        self.generic_client = GenericClient(
            ContainerInstanceManagementClient, subscription_id=subscription_id
        )
        self.client = self.generic_client.get_client()

        # Required parameters
        self.name = name
        self.resource_group_name = resource_group_name
        self.subscription_id = subscription_id

        # Optionnal parameters
        self.location = location
        self.image = image
        self.cpus = cpus
        self.mem = mem
        self.identity = identity
        self.restart_policy = restart_policy
        self.os_type = OperatingSystemTypes.linux
        self.command = command
        self.group = None
        self.instances = []
        self.tags = tags
        self.protected = True
        self.env_vars = []
        self.identity_name = identity_name
        self.container_name = "container-001"
        self.last_finish_time = None

        if env_vars:
            self.set_env_vars(env_vars)

        try:
            # container group already exists
            self._get_group()
            self.cpus = 0
            self.mem = 0
            self.state = "Accessed"
        except:

            # container group provisionned (before create methos call)
            self.state = "Provisionned"
            if self.identity_name:
                self.set_identity_by_name(self.identity_name)

    def _get_group(self):
        self.group = self.client.container_groups.get(
            self.resource_group_name, self.name
        )

        self.restart_policy = self.group.restart_policy
        self.identity = self.group.identity

        for instance in self.group.containers:
            self.instances.append(instance)

    def add_env_var(self, name, value):
        """
        add environement var
        
        Args:
            - name (str): env var name
            - value (str): env var value
        """
        self.env_vars.append(EnvironmentVariable(name=name, value=value))

    def set_env_vars(self, vars):
        """
        set env vars from dictionary
        
        Args:
            - vars (dict): env vars dictionary
        """
        for key, value in vars.items():
            self.add_env_var(key, value)

    def set_cpus(self, cpus):
        """
        set cores number
        
        Args:
            - cpus (int): 1-4 (actual limit for northeurope)
        """
        self.cpus = cpus

    def set_mem(self, mem):
        """
        set amount of memory in GB
        
        Args:
            - mem (int): 1-14 (actual limit for northeurope)
        """
        self.mem = mem

    def set_identity(self, identity):
        """
        set user managed identity
        
        Args:
            - identity (taz.auth.UserAssignedIdentity): container group identity
        """
        self.identity = identity.get_container_group_identity()

    def set_identity_by_name(self, identity_name, resource_group=None):
        """
        set identity by identity name instead of taz.auth.UserAssignedIdentity object (set_identity method)

        Args:
            - identity_name (str): user assigned identity name to affect
            - resource_group(None, str, optionnal): if not set, same as container group
        """
        if not resource_group:
            resource_group = self.resource_group_name
        self.identity_name = identity_name
        self.identity = UserAssignedIdentity(
            resource_group, self.identity_name, subscription_id=self.subscription_id
        ).get_container_group_identity()

    def set_command(self, command):
        """
        set command entry point of container
        
        Args:
            - command (str): entry point script of container
        """
        self.command = command

    def create(self):
        """
        Create the container group and instance
        """

        if self.state == "Accessed":
            raise (ContainerAlreadyExistsException(self.name))

        # Configure the container
        container_resource_requests = ResourceRequests(
            memory_in_gb=self.mem, cpu=self.cpus
        )
        container_resource_requirements = ResourceRequirements(
            requests=container_resource_requests
        )

        container = Container(
            name=self.container_name,
            image="{0}.azurecr.io/{1}:latest".format(
                self.image.container_registry.name, self.image.name
            ),
            resources=container_resource_requirements,
            environment_variables=self.env_vars,
            command=self.command,
        )

        group = ContainerGroup(
            location=self.location,
            containers=[container],
            os_type=self.os_type,
            restart_policy=self.restart_policy,
            image_registry_credentials=[
                self.image.container_registry.get_credentials()
            ],
            identity=self.identity,
            tags=self.tags,
        )

        self.client.container_groups.create_or_update(
            self.resource_group_name, self.name, group
        )

        self._get_group()
        self.state = "Created"

    def get_group(self):
        """
        Return container group attribute
        
        Returns:
            - azure.mgmt.containerinstance.models.ContainerGroup: container 
                group object
        """
        return self.group

    def get_instances(self):
        """
        Summary
        
        Returns:
            - [ azure.mgmt.containerinstance.models.Container ]: List of
                container instances
        """
        return self.instances

    def delete(self):
        """
        delete container group and his containers
        """
        self.client.container_groups.delete(self.resource_group_name, self.name)

    def list_logs(self):
        """list containers logs
        
        :return: return container logs
        :rtype: str
        """
        instances_logs = ""
        for instance in self.instances:
            instances_logs += "Logs for group '{}' for container '{}':\n".format(
                self.name, instance.name
            )
            try:
                instance_logs = self.client.container.list_logs(
                    self.resource_group_name, self.name, instance.name
                )
                instances_logs += "{0}".format(instance_logs.content) + "\n"
            except:
                instances_logs += "unavailable" + "\n"

        return instances_logs

    def save_logs(
        self,
        container,
        sas_url=None,
        sas_key=None,
        storage_account_name=None,
        storage_account=None,
    ):

        datas_group_logs = self.__str__()
        datas_container_logs = self.list_logs()

        if not self.last_finish_time == "None":
            name_group_log = "{}.{}.taz.aci.group.{}.log".format(
                self.last_start_time, self.last_finish_time, self.name
            )
            name_container_log = "{}.{}.taz.aci.container.{}.{}.log".format(
                self.last_start_time,
                self.last_finish_time,
                self.name,
                self.container_name,
            )
            blob_group_logs = Blob(
                container,
                name_group_log,
                sas_url=sas_url,
                sas_key=sas_key,
                storage_account_name=storage_account_name,
                storage_account=storage_account,
            )
            blob_container_logs = Blob(
                container,
                name_container_log,
                sas_url=sas_url,
                sas_key=sas_key,
                storage_account_name=storage_account_name,
                storage_account=storage_account,
            )
            blob_group_logs.write(datas_group_logs)
            blob_container_logs.write(datas_container_logs)

            return name_group_log, name_container_log
        else:
            return None

    def __str__(self):
        """
        prints container group as json string
        
        Returns:
            - str: json encoded as string
        """

        instances = []

        for instance in self.instances:
            events = []
            vars = []
            try:
                for var in instance.environment_variables:
                    if self.protected is False:
                        value = var.value
                    else:
                        value = "<protected>"
                    vars.append({var.name: value})
                for event in instance.instance_view.events:
                    events.append(
                        "{} {} {} {}".format(
                            event.first_timestamp,
                            event.last_timestamp,
                            event.name,
                            event.message,
                        )
                    )
                command = instance.command
                ports = instance.ports
                state = instance.instance_view.current_state.state
                start_time = str(instance.instance_view.current_state.start_time)
                finish_time = str(instance.instance_view.current_state.finish_time)
                self.last_finish_time = finish_time
                self.last_start_time = start_time
                exit_code = instance.instance_view.current_state.exit_code
                detail_status = instance.instance_view.current_state.detail_status
                memory = instance.resources.requests.memory_in_gb
                cpus = instance.resources.requests.cpu

            except:
                state = "unknown"
                start_time = "unknown"
                finish_time = "unknown"
                exit_code = "unknown"
                detail_status = "unknown"
                ports = []
                command = "unknown"
                cpus = "unknown"
                memory = "unknown"

            instances.append(
                {
                    "name": instance.name,
                    "image": instance.image,
                    "memory": memory,
                    "cpus": cpus,
                    "command": command,
                    "ports": ports,
                    "state": state,
                    "start_time": start_time,
                    "finish_time": finish_time,
                    "exit_code": exit_code,
                    "detail_status": detail_status,
                    "events": events,
                    "vars": vars,
                }
            )

        if self.group:
            group_name = self.group.name
        else:
            group_name = "None"

        return json.dumps(
            {
                "name": self.name,
                "identity": str(self.identity),
                "cpus": self.cpus,
                "memory": self.mem,
                "restart_policy": self.restart_policy,
                "state": self.state,
                "group_name": group_name,
                "instances": instances,
            },
            indent=4,
        )



#!/usr/bin/env python

from setuptools import setup, find_packages
import os
from taz._version import __version__

long_description = "SEE README file"

setup(
    name="cfa-taz",  # name your package
    packages=find_packages(exclude=["tests/config.py"]),
    version=__version__,
    description="High level API for azure",
    long_description=long_description,
    package_dir={"taz": "taz"},
    author="Christophe Fauchard",
    author_email="christophe.fauchard@gmail.com",
    license="MIT",  # MIT, GPL, BSD ??
    install_requires=["azure-keyvault-secrets>=4.1.0", "azure-identity>=1.3.0"],
    extras_require={
        "storage": [
            "azure-storage-blob>=12.3.0",
            "pandas>=0.25.3",
            "azure-cosmosdb-table>=1.0.6",
        ],
        "dev": ["pytest", "black", "sphinx", "sphinx-rtd-theme", "wheel", "twine"],
        "dls": ["azure-datalake-store>=0.0.48"],
        "aci": [
            "azure-mgmt-containerinstance>=1.5.0",
            "azure-mgmt-containerregistry>=2.8.0",
        ],
        "auth": [
            "adal>=1.2.2",
            "azure-mgmt-msi>=1.0.0",
            "azure-mgmt-containerinstance>=1.5.0",
        ],
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from taz.keyvault import KeyVault
import tests.config as cfg
import sys
import os


class KeyvaultTests(unittest.TestCase):

    def setUp(self):
        os.environ["AZURE_TENANT_ID"] = cfg.keyvault['tenant_id']
        os.environ["AZURE_CLIENT_ID"] = cfg.keyvault['client_id']
        os.environ["AZURE_CLIENT_SECRET"] = cfg.keyvault['client_secret']

        self.keyvault = KeyVault(cfg.keyvault['name'])
        print(self.keyvault.mode)

    def test_01_exists(self):
        self.assertTrue(self.keyvault is not None)

    def test_10_get_mode(self):
        self.assertTrue(
            self.keyvault.mode == "service_principal")

    def test_20_get_secret(self):
        secret = self.keyvault.get_secret(cfg.keyvault['secret_name'], '')
        self.assertTrue(secret is not None)

    def test_30_set_secret(self):
        self.keyvault.set_secret('testtazkey', 'testtazvalue')

    def test_40_get_secret(self):
        self.assertTrue(
            self.keyvault.get_secret('testtazkey').value == "testtazvalue")



if __name__ == '__main__':
    sys.argv.append('-v')
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class ContainerAlreadyExistsException(Exception):
    def __init__(self, container_name):
        self.container_name = container_name


class BlobAthenticationFailedException(Exception):
    def __init__(self, blob_name):
        self.blob_name = blob_name


class TableBatchExceedCapacityException(Exception):
    def __init__(self, storage_account_name, table_name):
        self.storage_account_name = storage_account_name
        self.table_name = table_name


class TableBatchDeleteForbiddenException(Exception):
    def __init__(self, storage_account_name, table_name):
        self.storage_account_name = storage_account_name
        self.table_name = table_name


class TableBatchDataframeTooLargeException(Exception):
    def __init__(self, dataframe_size):
        self.dataframe_size = dataframe_size
